@@ -4,12 +4,12 @@ __author__ = ["Alexander Willner", "Michael Belfrage"]
 __copyright__ = "2021 Alexander Willner & Michael Belfrage"
 __credits__ = ["Alexander Willner", "Michael Belfrage"]
 __license__ = "Apache License 2.0"
-__version__ = "0.0.11"
+__version__ = "0.0.12"
 __maintainer__ = ["Alexander Willner", "Michael Belfrage"]
 __email__ = "alex@willner.ws"
 __status__ = "Development"
 
-from things.api import (  # noqa
+from things.api import (  # noqa  isort:skip
     anytime,
     areas,
     canceled,
@@ -33,4 +33,5 @@ from things.api import (  # noqa
     trash,
     upcoming,
 )
+
 from things.database import Database  # noqa
