@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 Module implementing the Things API.
 
@@ -11,7 +9,6 @@ data structures. Whenever that happens, we define the new term here.
 
 import os
 from shlex import quote
-import sys
 from typing import Dict, List, Union
 
 from things.database import Database
@@ -42,7 +39,7 @@ def tasks(uuid=None, include_items=False, **kwargs):  # noqa: C901
         Include items contained within a task. These might include
         checklist items, headings, and to-dos.
 
-    type : {'to-do', 'heading', 'project', None}, optional
+    type : {'to-do', 'heading', 'project', None}, optional
         Only return a specific type of task:
         - `'to-do'`:    may have a checklist; may be in an area and have tags.
         - `'project'`:  may have to-dos and headings; may be in an area and
@@ -60,7 +57,7 @@ def tasks(uuid=None, include_items=False, **kwargs):  # noqa: C901
         Only include tasks matching that start value. If the argument is
         `None` (default), then include tasks with any start value.
 
-    area : str or bool or None, optional
+    area : str or bool or None, optional
         Any valid uuid of an area. Only include tasks matching that area.
         Special cases:
         - `area == False`, only include tasks _without_ an area.
@@ -88,21 +85,50 @@ def tasks(uuid=None, include_items=False, **kwargs):  # noqa: C901
         - `tag == True`, only include tasks _with_ tags.
         - `tag == None` (default), then include all tasks.
 
-    start_date : bool or None, optional
+    start_date : bool, str or None, optional
         - `start_date == False`, only include tasks _without_ a start date.
-        - `start_date == True`, only include tasks _with_ a start date.
+        - `start_date == True`, only include tasks with a start date.
+        - `start_date == 'future'`, only include tasks with a future start date.
+        - `start_date == 'past'`, only include tasks with a past start date.
+          Note: this includes today's date.
         - `start_date == None` (default), then include all tasks.
 
-    deadline : bool or None, optional
+    deadline : bool, str or None, optional
         - `deadline == False`, only include tasks _without_ a deadline.
         - `deadline == True`, only include tasks _with_ a deadline.
+        - `deadline == 'future'`, only include tasks with a future deadline.
+        - `deadline == 'past'`, only include tasks with a past deadline.
+          Note: this includes today's date.
         - `deadline == None` (default), then include all tasks.
 
-    trashed : bool or None, optional
+    deadline_suppressed : bool or None, optional
+        "deadline suppressed" is a technical term used in the database.
+        When tasks have an overdue deadline they show up in Today.
+        Some of us "suppress" some tasks, that is, move them out of
+        Today and into Inbox, Anytime, or Someday. For those moved tasks
+        the deadline is said to be suppressed.
+        - `deadline_suppressed == True`, only include tasks with deadlines
+          and whose deadlines have been suppressed.
+        - `deadline_suppressed == False`, include any task _except_ those
+          with suppressed deadlines.
+        - `deadline_suppressed == None` (default), include any task.
+
+    trashed : bool or None, optional, default False
+        - `trashed == False` (default), only include non-trashed tasks.
         - `trashed == True`, only include trashed tasks.
-        - `trashed == False`, (default), only include non-trashed tasks.
         - `trashed == None`, include both kind of tasks.
 
+    context_trashed : bool or None, optional, default False
+        Some tasks may be within a context of a project or a heading.
+        This manages how to handle such tasks. The other tasks are not
+        affected by this setting.
+
+        - `context_trashed == False` (default), for tasks within a context,
+           only return tasks whose context has _not_ been trashed.
+        - `context_trashed == True`, for tasks within a context,
+           only return tasks whose context has been trashed.
+        - `context_trashed == None`, include both kind of tasks.
+
     last : str, optional
         Limit returned tasks to tasks created within the last X days,
         weeks, or years. For example: '3d', '5w', or '1y'.
@@ -192,6 +218,7 @@ def tasks(uuid=None, include_items=False, **kwargs):  # noqa: C901
             project = task
             project["items"] = items = tasks(
                 project=project["uuid"],
+                context_trashed=None,
                 include_items=True,
                 database=database,
             )
@@ -202,6 +229,7 @@ def tasks(uuid=None, include_items=False, **kwargs):  # noqa: C901
             heading["items"] = tasks(
                 type="to-do",
                 heading=heading["uuid"],
+                context_trashed=None,
                 include_items=True,
                 database=database,
             )
@@ -465,30 +493,47 @@ def today(**kwargs):
     """
     Read Today's tasks into dicts.
 
-    Note: This might not produce desired results if the Things app hasn't
-    been opened yet today and the yellow "OK" button clicked for new tasks.
-    In general, you can assume that whatever state the Things app was in
-    when you last opened it, that's the state reflected here by the API.
+    Note: The Things database reflects the state of the Things app when
+    it was last opened. For the Today tasks that means the database
+    might not be up to date anymore if you didn't open the app recently.
+    To get around this limitation, we here make a prediction of what
+    tasks would show up in Today if you were to open the app right now.
+    This prediction does not include repeating tasks at this time.
 
     See `things.api.tasks` for details on the optional parameters.
     """
-    database = pop_database(kwargs)
-    if not database.was_modified_today():  # pragma: no cover
-        print(
-            "[NOTE] The results reflect the state of the Things app "
-            "when it was last run. If the results seem out of date, "
-            "then run the Things app and click the yellow 'OK' button "
-            "to update Today's to-dos and projects.",
-            file=sys.stderr,
-        )
-    return tasks(
+    regular_today_tasks = tasks(
         start_date=True,
         start="Anytime",
         index="todayIndex",
-        database=database,
         **kwargs,
     )
 
+    # Predictions of new to-dos indicated by a yellow dot in the app.
+
+    unconfirmed_scheduled_tasks = tasks(
+        start_date="past",
+        start="Someday",
+        index="todayIndex",
+        **kwargs,
+    )
+
+    unconfirmed_overdue_tasks = tasks(
+        start_date=False,
+        deadline="past",
+        deadline_suppressed=False,
+        **kwargs,
+    )
+
+    result = [
+        *regular_today_tasks,
+        *unconfirmed_scheduled_tasks,
+        *unconfirmed_overdue_tasks,
+    ]
+    result.sort(key=lambda task: (task["today_index"], task["start_date"]))
+
+    return result
+
 
 def upcoming(**kwargs):
     """
@@ -499,7 +544,7 @@ def upcoming(**kwargs):
 
     For details on parameters, see `things.api.tasks`.
     """
-    return tasks(start_date=True, start="Someday", **kwargs)
+    return tasks(start_date="future", start="Someday", **kwargs)
 
 
 def anytime(**kwargs):
@@ -537,7 +582,12 @@ def trash(**kwargs):
 
     See `things.api.tasks` for details on the optional parameters.
     """
-    return tasks(trashed=True, status=kwargs.pop("status", None), **kwargs)
+    return tasks(
+        trashed=True,
+        context_trashed=kwargs.pop("context_trashed", None),
+        status=kwargs.pop("status", None),
+        **kwargs,
+    )
 
 
 # Filter by various task properties
@@ -634,14 +684,11 @@ def token(**kwargs) -> Union[str, None]:
 
 
 def link(uuid):
-    """
-    Returns a things:///show?id=uuid link.
-    """
-
+    """Return a things:///show?id=uuid link."""
     return f"things:///show?id={uuid}"
 
 
-def show(uuid):
+def show(uuid):  # noqa
     """
     Show a certain uuid in the Things app.
 
@@ -663,7 +710,7 @@ def show(uuid):
 
 
 def pop_database(kwargs):
-    """instantiate non-default database from `kwargs` if provided"""
+    """Instantiate non-default database from `kwargs` if provided."""
     filepath = kwargs.pop("filepath", None)
     database = kwargs.pop("database", None)
     print_sql = kwargs.pop("print_sql", False)
