@@ -1,14 +1,9 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
-
 """Read from the Things SQLite database using SQL queries."""
 
-import datetime
 import os
 import plistlib
 import re
 import sqlite3
-import sys
 from textwrap import dedent
 
 
@@ -44,6 +39,10 @@ TRASHED_TO_FILTER = {True: "trashed = 1", False: "trashed = 0"}
 
 TYPE_TO_FILTER = {"to-do": "type = 0", "project": "type = 1", "heading": "type = 2"}
 
+# Dates
+
+DATES = ("future", "past", True, False)
+
 # Indices
 
 INDICES = ("index", "todayIndex")
@@ -77,20 +76,13 @@ TABLE_TASKTAG = "TMTaskTag"
 TABLE_SETTINGS = "TMSettings"
 
 # --------------------------------------------------
-# Dates
+# Date Columns
 # --------------------------------------------------
 
-# Columns
 DATE_CREATED = "creationDate"
 DATE_DEADLINE = "dueDate"
 DATE_MODIFIED = "userModificationDate"
 DATE_START = "startDate"
-DATE_STOP = "stopDate"
-
-# Filters
-IS_SCHEDULED = f"{DATE_START} IS NOT NULL"
-IS_NOT_SCHEDULED = f"{DATE_START} IS NULL"
-IS_DEADLINE = f"{DATE_DEADLINE} IS NOT NULL"
 
 # --------------------------------------------------
 # Various filters
@@ -112,17 +104,29 @@ IS_ANYTIME = START_TO_FILTER["Anytime"]
 IS_SOMEDAY = START_TO_FILTER["Someday"]
 
 # Repeats
-IS_RECURRING = "recurrenceRule IS NOT NULL"
 IS_NOT_RECURRING = "recurrenceRule IS NULL"
-RECURRING_IS_NOT_PAUSED = "instanceCreationPaused = 0"
-RECURRING_HAS_NEXT_STARTDATE = "nextInstanceStartDate IS NOT NULL"
 
-# Trashed
-IS_NOT_TRASHED = TRASHED_TO_FILTER[False]
+# Trash
 IS_TRASHED = TRASHED_TO_FILTER[True]
 
+# --------------------------------------------------
+# Fields and filters not yet used in the implementation.
+# This information might be of relevance in the future.
+# --------------------------------------------------
+#
+# DATE_START = "startDate"
+# DATE_STOP = "stopDate"
+# IS_SCHEDULED = f"{DATE_START} IS NOT NULL"
+# IS_NOT_SCHEDULED = f"{DATE_START} IS NULL"
+# IS_DEADLINE = f"{DATE_DEADLINE} IS NOT NULL"
+# RECURRING_IS_NOT_PAUSED = "instanceCreationPaused = 0"
+# IS_RECURRING = "recurrenceRule IS NOT NULL"
+# RECURRING_HAS_NEXT_STARTDATE = ("nextInstanceStartDate IS NOT NULL")
+# IS_NOT_TRASHED = TRASHED_TO_FILTER[False]
 
 # pylint: disable=R0904,R0902
+
+
 class Database:
     """
     Access Things SQL database.
@@ -144,13 +148,14 @@ class Database:
 
     # pylint: disable=R0913
     def __init__(self, filepath=None, print_sql=False):
+        """Set up the database."""
         self.filepath = (
             filepath
             or os.getenv(ENVIRONMENT_VARIABLE_WITH_FILEPATH)
             or DEFAULT_FILEPATH
         )
         self.print_sql = print_sql
-        if print_sql:
+        if self.print_sql:
             self.execute_query_count = 0
 
         # Automated migration to new database location in Things 3.12.6/3.13.1
@@ -177,22 +182,30 @@ class Database:
         tag=None,
         start_date=None,
         deadline=None,
-        trashed: bool = False,
+        deadline_suppressed=None,
+        trashed=False,
+        context_trashed=False,
         last=None,
         search_query=None,
         index="index",
         count_only=False,
     ):
         """Get tasks. See `things.api.tasks` for details on parameters."""
+        if uuid:
+            return self.get_task_by_uuid(uuid, count_only=count_only)
 
         # Overwrites
         start = start and start.title()
 
         # Validation
+        validate("deadline", deadline, [None] + list(DATES))  # type: ignore
+        validate("deadline_suppressed", deadline_suppressed, [None, True, False])  # type: ignore
         validate("start", start, [None] + list(START_TO_FILTER))  # type: ignore
+        validate("start_date", start_date, [None] + list(DATES))  # type: ignore
         validate("status", status, [None] + list(STATUS_TO_FILTER))  # type: ignore
         validate("trashed", trashed, [None] + list(TRASHED_TO_FILTER))  # type: ignore
         validate("type", type, [None] + list(TYPE_TO_FILTER))  # type: ignore
+        validate("context_trashed", context_trashed, [None, True, False])
         validate("index", index, list(INDICES))
         validate_offset("last", last)
 
@@ -200,42 +213,44 @@ class Database:
             valid_tags = self.get_tags(titles_only=True)
             validate("tag", tag, [None] + list(valid_tags))
 
-        if uuid:
-            if count_only is False and not self.get_tasks(uuid=uuid, count_only=True):
-                raise ValueError(f"No such task uuid found: {uuid!r}")
-
         # Query
         # TK: might consider executing SQL with parameters instead.
         # See: https://docs.python.org/3/library/sqlite3.html#sqlite3.Cursor.execute
 
-        if uuid:
-            where_predicate = f"TRUE {make_filter('TASK.uuid', uuid)}"
-            order_predicate = None
-        else:
-            start_filter = START_TO_FILTER.get(start, "")
-            status_filter = STATUS_TO_FILTER.get(status, "")
-            trashed_filter = TRASHED_TO_FILTER.get(trashed, "")
-            type_filter = TYPE_TO_FILTER.get(type, "")
+        start_filter = START_TO_FILTER.get(start, "")
+        status_filter = STATUS_TO_FILTER.get(status, "")
+        trashed_filter = TRASHED_TO_FILTER.get(trashed, "")
+        type_filter = TYPE_TO_FILTER.get(type, "")
 
-            where_predicate = f"""
-                TASK.{IS_NOT_RECURRING}
-                AND (PROJECT.title IS NULL OR PROJECT.{IS_NOT_TRASHED})
-                AND (HEADPROJ.title IS NULL OR HEADPROJ.{IS_NOT_TRASHED})
-                {trashed_filter and f"AND TASK.{trashed_filter}"}
-                {type_filter and f"AND TASK.{type_filter}"}
-                {start_filter and f"AND TASK.{start_filter}"}
-                {status_filter and f"AND TASK.{status_filter}"}
-                {make_filter('TASK.uuid', uuid)}
-                {make_filter("TASK.area", area)}
-                {make_filter("TASK.project", project)}
-                {make_filter("TASK.actionGroup", heading)}
-                {make_filter("TASK.startDate", start_date)}
-                {make_filter(f"TASK.{DATE_DEADLINE}", deadline)}
-                {make_filter("TAG.title", tag)}
-                {make_date_filter(f"TASK.{DATE_CREATED}", last)}
-                {make_search_filter(search_query)}
-                """
-            order_predicate = f'TASK."{index}"'
+        # Sometimes a task is _not_ set to trashed, but its context
+        # (project or heading it is contained within) is set to trashed.
+        # In those cases, the task wouldn't show up in any app view
+        # except for "Trash".
+        project_trashed_filter = make_truthy_filter("PROJECT.trashed", context_trashed)
+        project_of_heading_trashed_filter = make_truthy_filter(
+            "PROJECT_OF_HEADING.trashed", context_trashed
+        )
+
+        where_predicate = f"""
+            TASK.{IS_NOT_RECURRING}
+            {trashed_filter and f"AND TASK.{trashed_filter}"}
+            {project_trashed_filter}
+            {project_of_heading_trashed_filter}
+            {type_filter and f"AND TASK.{type_filter}"}
+            {start_filter and f"AND TASK.{start_filter}"}
+            {status_filter and f"AND TASK.{status_filter}"}
+            {make_filter('TASK.uuid', uuid)}
+            {make_filter("TASK.area", area)}
+            {make_filter("TASK.project", project)}
+            {make_filter("TASK.actionGroup", heading)}
+            {make_filter("TASK.dueDateSuppressionDate", deadline_suppressed)}
+            {make_filter("TAG.title", tag)}
+            {make_date_filter(f"TASK.{DATE_START}", start_date)}
+            {make_date_filter(f"TASK.{DATE_DEADLINE}", deadline)}
+            {make_date_range_filter(f"TASK.{DATE_CREATED}", last)}
+            {make_search_filter(search_query)}
+            """
+        order_predicate = f'TASK."{index}"'
 
         sql_query = make_tasks_sql_query(where_predicate, order_predicate)
 
@@ -244,9 +259,23 @@ class Database:
 
         return self.execute_query(sql_query)
 
+    def get_task_by_uuid(self, uuid, count_only=False):
+        """Get a task by uuid. Raise `ValueError` if not found."""
+        where_predicate = "TASK.uuid = ?"
+        sql_query = make_tasks_sql_query(where_predicate)
+        parameters = (uuid,)
+
+        if count_only:
+            return self.get_count(sql_query, parameters)
+
+        result = self.execute_query(sql_query, parameters)
+        if not result:
+            raise ValueError(f"No such task uuid found: {uuid!r}")
+
+        return result
+
     def get_areas(self, uuid=None, tag=None, count_only=False):
         """Get areas. See `api.areas` for details on parameters."""
-
         # Validation
         if tag is not None:
             valid_tags = self.get_tags(titles_only=True)
@@ -315,7 +344,6 @@ class Database:
 
     def get_tags(self, title=None, area=None, task=None, titles_only=False):
         """Get tags. See `api.tags` for details on parameters."""
-
         # Validation
         if title is not None:
             valid_titles = self.get_tags(titles_only=True)
@@ -345,7 +373,7 @@ class Database:
         return self.execute_query(sql_query)
 
     def get_tags_of_task(self, task_uuid):
-        """Get tag titles of task"""
+        """Get tag titles of task."""
         sql_query = f"""
             SELECT
                 TAG.title
@@ -362,7 +390,7 @@ class Database:
         )
 
     def get_tags_of_area(self, area_uuid):
-        """Get tag titles for area"""
+        """Get tag titles for area."""
         sql_query = f"""
             SELECT
                 AREA.title
@@ -380,7 +408,6 @@ class Database:
 
     def get_version(self):
         """Get Things Database version."""
-
         sql_query = f"SELECT value FROM {TABLE_META} WHERE key = 'databaseVersion'"
         result = self.execute_query(sql_query, row_factory=list_factory)
         plist_bytes = result[0].encode()
@@ -389,7 +416,6 @@ class Database:
     # pylint: disable=R1710
     def get_url_scheme_auth_token(self):
         """Get the Things URL scheme authentication token."""
-
         sql_query = f"""
             SELECT
                 uriSchemeAuthenticationToken
@@ -401,21 +427,25 @@ class Database:
         rows = self.execute_query(sql_query, row_factory=list_factory)
         return rows[0]
 
-    def get_count(self, sql):
+    def get_count(self, sql_query, parameters=()):
         """Count number of results."""
-        sql_query = f"""SELECT COUNT(uuid) FROM (\n{sql}\n)"""
-        rows = self.execute_query(sql_query, row_factory=list_factory)
+        count_sql_query = f"""SELECT COUNT(uuid) FROM (\n{sql_query}\n)"""
+        rows = self.execute_query(
+            count_sql_query, row_factory=list_factory, parameters=parameters
+        )
         return rows[0]
 
     # noqa todo: add type hinting for resutl (List[Tuple[str, Any]]?)
     def execute_query(self, sql_query, parameters=(), row_factory=None):
-        """Run the actual SQL query"""
-        if self.debug is True:
-            print(self.filepath)
-            print(sql_query)
-
-        if self.print_sql:
+        """Run the actual SQL query."""
+        if self.print_sql or self.debug:
+            if not hasattr(self, "execute_query_count"):
+                # This is needed for historical `self.debug`.
+                # TK: might consider removing `debug` flag.
+                self.execute_query_count = 0
             self.execute_query_count += 1
+            if self.debug:
+                print(f"/* Filepath {self.filepath!r} */")
             print(f"/* Query {self.execute_query_count} */")
             if parameters:
                 print(f"/* Parameters: {parameters!r} */")
@@ -423,44 +453,22 @@ class Database:
             print(prettify_sql(sql_query))
             print()
 
-        try:
-            # "ro" means read-only
-            # See: https://sqlite.org/uri.html#recognized_query_parameters
-            uri = f"file:{self.filepath}?mode=ro"
-            connection = sqlite3.connect(uri, uri=True)  # pylint: disable=E1101
-            connection.row_factory = row_factory or dict_factory
-            cursor = connection.cursor()
-            cursor.execute(sql_query, parameters)
-            rows = cursor.fetchall()
-            if self.debug:
-                for row in rows:
-                    print(row)
-            return rows
-        except sqlite3.OperationalError as error:  # pylint: disable=E1101
-            print(f"Could not query the database at: {self.filepath}.")
-            print(f"Details: {error}.")
-            sys.exit(2)
+        # "ro" means read-only
+        # See: https://sqlite.org/uri.html#recognized_query_parameters
+        uri = f"file:{self.filepath}?mode=ro"
+        connection = sqlite3.connect(uri, uri=True)  # pylint: disable=E1101
+        connection.row_factory = row_factory or dict_factory
+        cursor = connection.cursor()
+        cursor.execute(sql_query, parameters)
 
-    # -------- Utility methods --------
-
-    def last_modified(self):
-        """Get last modified time."""
-        mtime_seconds = os.path.getmtime(self.filepath)
-        return datetime.datetime.fromtimestamp(mtime_seconds)
-
-    def was_modified_today(self):
-        """Was task modified today?"""
-        last_modified_date = self.last_modified().date()
-        todays_date = datetime.datetime.now().date()
-        return last_modified_date >= todays_date
+        return cursor.fetchall()
 
 
 # Helper functions
 
 
 def make_tasks_sql_query(where_predicate=None, order_predicate=None):
-    """Make SQL query for Task table"""
-
+    """Make SQL query for Task table."""
     where_predicate = where_predicate or "TRUE"
     order_predicate = order_predicate or 'TASK."index"'
 
@@ -527,14 +535,15 @@ def make_tasks_sql_query(where_predicate=None, order_predicate=None):
             LEFT OUTER JOIN
                 {TABLE_TASK} HEADING ON TASK.actionGroup = HEADING.uuid
             LEFT OUTER JOIN
-                {TABLE_TASK} HEADPROJ ON HEADING.project = HEADPROJ.uuid
+                {TABLE_TASK} PROJECT_OF_HEADING
+                ON HEADING.project = PROJECT_OF_HEADING.uuid
             LEFT OUTER JOIN
                 {TABLE_TASKTAG} TAGS ON TASK.uuid = TAGS.tasks
             LEFT OUTER JOIN
                 {TABLE_TAG} TAG ON TAGS.tags = TAG.uuid
             LEFT OUTER JOIN
                 {TABLE_CHECKLIST_ITEM} CHECKLIST_ITEM
-                ON CHECKLIST_ITEM.task = TASK.uuid
+                ON TASK.uuid = CHECKLIST_ITEM.task
             WHERE
                 {where_predicate}
             ORDER BY
@@ -585,14 +594,13 @@ def escape_string(string):
 
 
 def list_factory(_cursor, row):
-    """
-    Convert SQL selects of one column into a list.
-    """
+    """Convert SQL selects of one column into a list."""
     return row[0]
 
 
 def make_filter(column, value):
-    """Return SQL filter 'AND {column} = "{value}"'.
+    """
+    Return SQL filter 'AND {column} = "{value}"'.
 
     Special handling if `value` is `bool` or `None`.
 
@@ -611,7 +619,6 @@ def make_filter(column, value):
     ''
     """
     default = f"AND {column} = '{escape_string(str(value))}'"
-    # special options
     return {
         None: "",
         False: f"AND {column} IS NULL",
@@ -619,9 +626,55 @@ def make_filter(column, value):
     }.get(value, default)
 
 
-def make_date_filter(date_column, offset):
+def make_date_filter(date_column: str, value) -> str:
     """
-    Returns SQL filter to limit the date range of the SQL query.
+    Return a SQL filter for date columns.
+
+    Parameters
+    ----------
+    date_column : str
+        Name of the column that has date information on a task.
+
+    value : bool, 'future', 'past', or None
+        `True` or `False` indicates whether a date is set or not.
+        `'future'` or `'past'` indicates a date in the future or past.
+        `None` indicates any value.
+
+    Returns
+    -------
+    str
+        A date filter for the SQL query. If `value == None`, then
+        return the empty string.
+
+    Examples
+    --------
+    >>> make_date_filter('start_date', True)
+    'AND start_date IS NOT NULL'
+
+    >>> make_date_filter('start_date', False)
+    'AND start_date IS NULL'
+
+    >>> make_date_filter('start_date', 'future')
+    "AND datetime(start_date, 'unixepoch') > datetime('now')"
+
+    >>> make_date_filter('created', None)
+    ''
+
+    """
+    if value is None:
+        return ""
+
+    if isinstance(value, bool):
+        return make_filter(date_column, value)
+
+    validate("value", value, ["future", "past"])
+    operator = ">" if value == "future" else "<="
+    return f"AND datetime({date_column}, 'unixepoch') {operator} datetime('now')"
+
+
+def make_date_range_filter(date_column, offset) -> str:
+    """
+    Return a SQL filter to limit a date to last X days, weeks, or years.
 
     Parameters
     ----------
@@ -641,20 +694,16 @@ def make_date_filter(date_column, offset):
 
     Examples
     --------
-    >>> make_date_filter('created', '3d')
-    "AND datetime(created, 'unixepoch', 'localtime') > datetime('now', '-3 days')"
+    >>> make_date_range_filter('created', '3d')
+    "AND datetime(created, 'unixepoch') > datetime('now', '-3 days')"
 
-    >>> make_date_filter('created', None)
+    >>> make_date_range_filter('created', None)
     ''
-
     """
-
-    # Offset not specified
     if offset is None:
         return ""
 
     validate_offset("offset", offset)
-
     number, suffix = int(offset[:-1]), offset[-1]
 
     if suffix == "d":
@@ -664,16 +713,47 @@ def make_date_filter(date_column, offset):
     elif suffix == "y":
         modifier = f"-{number} years"
 
-    column_datetime = f"datetime({date_column}, 'unixepoch', 'localtime')"
-    offset_datetime = f"datetime('now', 'localtime', '{modifier}')"  # type: ignore
+    column_datetime = f"datetime({date_column}, 'unixepoch')"
+    offset_datetime = f"datetime('now', '{modifier}')"  # type: ignore
 
     return f"AND {column_datetime} > {offset_datetime}"
 
 
+def make_truthy_filter(column: str, value) -> str:
+    """
+    Return a SQL filter that matches if a column is truthy or falsy.
+
+    Truthy means TRUE. Falsy means FALSE or NULL. This is akin
+    to how Python defines it natively.
+
+    Passing in `value == None` returns the empty string.
+
+    Examples
+    --------
+    >>> make_truthy_filter('PROJECT.trashed', True)
+    'AND PROJECT.trashed'
+
+    >>> make_truthy_filter('PROJECT.trashed', False)
+    'AND NOT IFNULL(PROJECT.trashed, 0)'
+
+    >>> make_truthy_filter('PROJECT.trashed', None)
+    ''
+    """
+    if value is None:
+        return ""
+
+    if value:
+        return f"AND {column}"
+
+    return f"AND NOT IFNULL({column}, 0)"
+
+
 def make_search_filter(query: str) -> str:
     """
-    Example
-    -------
+    Return a SQL filter to search tasks by a string query.
+
+    Example:
+    --------
     >>> make_search_filter('dinner')
     'AND (
         TASK.title LIKE "%dinner%"
@@ -695,9 +775,7 @@ def make_search_filter(query: str) -> str:
 
 
 def prettify_sql(sql_query):
-    """
-    Make a SQL query easier to read for humans.
-    """
+    """Make a SQL query easier to read for humans."""
     # remove indentation and leading and trailing whitespace
     result = dedent(sql_query).strip()
     # remove empty lines
@@ -707,6 +785,7 @@ def prettify_sql(sql_query):
 def validate(parameter, argument, valid_arguments):
     """
     For a given parameter, check if its argument type is valid.
+
     If not, then raise `ValueError`.
 
     Examples
@@ -735,6 +814,7 @@ def validate(parameter, argument, valid_arguments):
 def validate_offset(parameter, argument):
     """
     For a given offset parameter, check if its argument is valid.
+
     If not, then raise `ValueError`.
 
     Examples
